@@ -263,6 +263,7 @@ void Executor::doBuild()
     QBS_CHECK(!m_project->buildData->evaluationContext);
     m_project->buildData->evaluationContext = std::make_shared<RulesEvaluationContext>(m_logger);
     m_evalContext = m_project->buildData->evaluationContext;
+    m_progressObserver->addScriptEngine(m_evalContext->engine());
 
     m_elapsedTimeRules = m_elapsedTimeScanners = m_elapsedTimeInstalling = 0;
     m_evalContext->engine()->enableProfiling(m_buildOptions.logElapsedTime());
@@ -535,8 +536,8 @@ void Executor::executeRuleNode(RuleNode *ruleNode)
 
     QBS_CHECK(!m_evalContext->engine()->isActive());
 
-    RuleNode::ApplicationResult result;
-    ruleNode->apply(m_logger, m_productsByName, m_projectsByName, &result);
+    const RuleNode::ApplicationResult result = ruleNode->apply(
+        m_logger, m_productsByName, m_projectsByName);
     updateLeaves(result.createdArtifacts);
     updateLeaves(result.invalidatedArtifacts);
     m_artifactsRemovedFromDisk << result.removedArtifacts;
