@@ -211,7 +211,7 @@ void ProcessCommandExecutor::cancel(const qbs::ErrorInfo &reason)
 QString ProcessCommandExecutor::filterProcessOutput(const QByteArray &_output,
         const QString &filterFunctionSource)
 {
-    QString output = QString::fromLocal8Bit(_output);
+    QString output = !_output.isNull() ? QString::fromLocal8Bit(_output) : QStringLiteral("");
     if (filterFunctionSource.isEmpty())
         return output;
 
@@ -291,7 +291,7 @@ void ProcessCommandExecutor::getProcessOutput(bool stdOut, ProcessResult &result
     } else {
         if (!contentString.isEmpty() && contentString.endsWith(QLatin1Char('\n')))
             contentString.chop(1);
-        *target = contentString.split(QLatin1Char('\n'), QBS_SKIP_EMPTY_PARTS);
+        *target = contentString.split(QLatin1Char('\n'), Qt::SkipEmptyParts);
     }
 }
 
