@@ -166,25 +166,28 @@ void BuiltinDeclarations::insert(const ItemDeclaration &decl)
 
 static PropertyDeclaration conditionProperty()
 {
-    return PropertyDeclaration(StringConstants::conditionProperty(), PropertyDeclaration::Boolean,
-                               StringConstants::trueValue());
+    return {
+        StringConstants::conditionProperty(),
+        PropertyDeclaration::Boolean,
+        StringConstants::trueValue()};
 }
 
 static PropertyDeclaration alwaysRunProperty()
 {
-    return PropertyDeclaration(StringConstants::alwaysRunProperty(), PropertyDeclaration::Boolean,
-                               StringConstants::falseValue());
+    return {
+        StringConstants::alwaysRunProperty(),
+        PropertyDeclaration::Boolean,
+        StringConstants::falseValue()};
 }
 
 static PropertyDeclaration nameProperty()
 {
-    return PropertyDeclaration(StringConstants::nameProperty(), PropertyDeclaration::String);
+    return {StringConstants::nameProperty(), PropertyDeclaration::String};
 }
 
 static PropertyDeclaration buildDirProperty()
 {
-    return PropertyDeclaration(StringConstants::buildDirectoryProperty(),
-                               PropertyDeclaration::Path);
+    return {StringConstants::buildDirectoryProperty(), PropertyDeclaration::Path};
 }
 
 static PropertyDeclaration prepareScriptProperty()
@@ -244,11 +247,11 @@ void BuiltinDeclarations::addDependsItem()
                                 PropertyDeclaration::StringList);
     item << PropertyDeclaration(StringConstants::limitToSubProjectProperty(),
                                 PropertyDeclaration::Boolean, StringConstants::falseValue());
-    item << PropertyDeclaration(StringConstants::multiplexConfigurationIdsProperty(),
-                                PropertyDeclaration::StringList, QString(),
-                                PropertyDeclaration::ReadOnlyFlag);
-    item << PropertyDeclaration(StringConstants::enableFallbackProperty(),
-                                PropertyDeclaration::Boolean, StringConstants::trueValue());
+    item << PropertyDeclaration(
+        StringConstants::multiplexConfigurationIdsProperty(),
+        PropertyDeclaration::StringList,
+        QString(),
+        PropertyDeclaration::ReadOnlyFlag);
     insert(item);
 }
 
@@ -258,7 +261,6 @@ void BuiltinDeclarations::addExportItem()
     item << PropertyDeclaration(StringConstants::prefixMappingProperty(),
                                 PropertyDeclaration::Variant);
     auto allowedChildTypes = item.allowedChildTypes();
-    allowedChildTypes.insert(ItemType::Parameters);
     allowedChildTypes.insert(ItemType::Properties);
     item.setAllowedChildTypes(allowedChildTypes);
     insert(item);
@@ -279,7 +281,12 @@ void BuiltinDeclarations::addFileTaggerItem()
 void BuiltinDeclarations::addGroupItem()
 {
     ItemDeclaration item(ItemType::Group);
-    item.setAllowedChildTypes({ ItemType::Group });
+    item.setAllowedChildTypes(
+        {ItemType::Depends,
+         ItemType::FileTagger,
+         ItemType::Group,
+         ItemType::Rule,
+         ItemType::Scanner});
     item << conditionProperty();
     item << PropertyDeclaration(StringConstants::nameProperty(), PropertyDeclaration::String,
                                 QString(), PropertyDeclaration::PropertyNotAvailableInConfig);
@@ -325,26 +332,30 @@ void BuiltinDeclarations::addModuleProviderItem()
 {
     ItemDeclaration item(ItemType::ModuleProvider);
     item << nameProperty()
+         << conditionProperty()
          << PropertyDeclaration(QStringLiteral("outputBaseDir"), PropertyDeclaration::String)
+         << PropertyDeclaration(StringConstants::isEagerProperty(),
+                                PropertyDeclaration::Boolean,
+                                StringConstants::trueValue())
+         << PropertyDeclaration(StringConstants::moduleNameProperty(), PropertyDeclaration::String)
          << PropertyDeclaration(QStringLiteral("relativeSearchPaths"),
                                 PropertyDeclaration::StringList);
+    item.setAllowedChildTypes({ItemType::PropertyOptions, ItemType::Probe});
     insert(item);
 }
 
 ItemDeclaration BuiltinDeclarations::moduleLikeItem(ItemType type)
 {
     ItemDeclaration item(type);
-    item.setAllowedChildTypes(ItemDeclaration::TypeNames()
-            << ItemType::Group
-            << ItemType::Depends
-            << ItemType::FileTagger
-            << ItemType::JobLimit
-            << ItemType::Rule
-            << ItemType::Parameter
-            << ItemType::Probe
-            << ItemType::PropertyOptions
-            << ItemType::Scanner);
-    item << nameProperty();
+    item.setAllowedChildTypes({ItemType::Depends, ItemType::FileTagger, ItemType::Group,
+                               ItemType::JobLimit, ItemType::Parameter, ItemType::Parameters,
+                               ItemType::Probe, ItemType::PropertyOptions,
+                               ItemType::Rule, ItemType::Scanner});
+    PropertyDeclaration nameDecl = nameProperty();
+    PropertyDeclaration::Flags nameFlags = nameDecl.flags();
+    nameFlags |= PropertyDeclaration::ReadOnlyFlag;
+    nameDecl.setFlags(nameFlags);
+    item << nameDecl;
     item << conditionProperty();
     PropertyDeclaration setupBuildEnvDecl(StringConstants::setupBuildEnvironmentProperty(),
                                           PropertyDeclaration::Variant, QString(),
@@ -383,16 +394,17 @@ void BuiltinDeclarations::addProbeItem()
 void BuiltinDeclarations::addProductItem()
 {
     ItemDeclaration item(ItemType::Product);
-    item.setAllowedChildTypes(ItemDeclaration::TypeNames()
-            << ItemType::Depends
-            << ItemType::Group
-            << ItemType::FileTagger
-            << ItemType::JobLimit
-            << ItemType::Export
-            << ItemType::Probe
-            << ItemType::Profile
-            << ItemType::PropertyOptions
-            << ItemType::Rule);
+    item.setAllowedChildTypes(
+        {ItemType::Depends,
+         ItemType::Export,
+         ItemType::FileTagger,
+         ItemType::Group,
+         ItemType::JobLimit,
+         ItemType::Probe,
+         ItemType::Profile,
+         ItemType::PropertyOptions,
+         ItemType::Rule,
+         ItemType::Scanner});
     item << conditionProperty();
     item << PropertyDeclaration(StringConstants::typeProperty(), PropertyDeclaration::StringList,
                                 StringConstants::emptyArrayValue());
@@ -433,6 +445,8 @@ void BuiltinDeclarations::addProductItem()
     item << PropertyDeclaration(StringConstants::multiplexConfigurationIdProperty(),
                                 PropertyDeclaration::String, QString(),
                                 PropertyDeclaration::ReadOnlyFlag);
+    item << PropertyDeclaration(StringConstants::qbsModuleProviders(),
+                                PropertyDeclaration::StringList);
     insert(item);
 }
 
@@ -473,6 +487,8 @@ void BuiltinDeclarations::addProjectItem()
     item << PropertyDeclaration(StringConstants::qbsSearchPathsProperty(),
                                 PropertyDeclaration::StringList,
                                 QString(), PropertyDeclaration::PropertyNotAvailableInConfig);
+    item << PropertyDeclaration(StringConstants::qbsModuleProviders(),
+                                PropertyDeclaration::StringList);
     insert(item);
 }
 
