@@ -40,13 +40,24 @@
 #ifndef QBS_PROPERTYDECLARATION_H
 #define QBS_PROPERTYDECLARATION_H
 
+#include <tools/deprecationwarningmode.h>
+
 #include <QtCore/qshareddata.h>
 #include <QtCore/qstring.h>
 
+QT_BEGIN_NAMESPACE
+class QVariant;
+QT_END_NAMESPACE
+
 namespace qbs {
+class CodeLocation;
+class ErrorInfo;
 namespace Internal {
 class DeprecationInfo;
 class PropertyDeclarationData;
+class Item;
+class LoaderState;
+class Logger;
 
 class PropertyDeclaration
 {
@@ -112,11 +123,27 @@ public:
     bool isExpired() const;
     const DeprecationInfo &deprecationInfo() const;
     void setDeprecationInfo(const DeprecationInfo &deprecationInfo);
+    ErrorInfo checkForDeprecation(DeprecationWarningMode mode, const CodeLocation &loc,
+                                  Logger &logger) const;
+
+    static QVariant convertToPropertyType(
+            const QVariant &v, Type t, const QStringList &namePrefix, const QString &key);
+    QVariant typedNullValue() const;
+
+    bool shouldCheckAllowedValues() const;
+    void checkAllowedValues(
+        const QVariant &value,
+        const CodeLocation &loc,
+        const QString &key,
+        LoaderState &loaderState) const;
 
 private:
     QSharedDataPointer<PropertyDeclarationData> d;
 };
 
+void checkPropertyDeclarations(Item *topLevelItem, LoaderState &loaderState);
+
+
 } // namespace Internal
 } // namespace qbs
 
