@@ -40,6 +40,10 @@
 #ifndef QBSQTTOOLS_H
 #define QBSQTTOOLS_H
 
+#include <tools/qbsassert.h>
+#include <tools/porting.h>
+#include <tools/stlutils.h>
+
 #include <QtCore/qhash.h>
 #include <QtCore/qstringlist.h>
 #include <QtCore/qtextstream.h>
@@ -51,19 +55,7 @@ QT_BEGIN_NAMESPACE
 class QProcessEnvironment;
 QT_END_NAMESPACE
 
-#if (QT_VERSION < QT_VERSION_CHECK(5, 14, 0))
-#define QBS_SKIP_EMPTY_PARTS QString::SkipEmptyParts
-#else
-#define QBS_SKIP_EMPTY_PARTS Qt::SkipEmptyParts
-#endif
-
 namespace std {
-#if (QT_VERSION < QT_VERSION_CHECK(5, 14, 0))
-template<> struct hash<QString> {
-    std::size_t operator()(const QString &s) const { return qHash(s); }
-};
-#endif
-
 template<typename T1, typename T2> struct hash<std::pair<T1, T2>>
 {
     size_t operator()(const pair<T1, T2> &x) const
@@ -71,18 +63,91 @@ template<typename T1, typename T2> struct hash<std::pair<T1, T2>>
         return std::hash<T1>()(x.first) ^ std::hash<T2>()(x.second);
     }
 };
+
+template <typename... Ts>
+struct hash<std::tuple<Ts...>>
+{
+private:
+    template<std::size_t... Ns>
+    static size_t helper(std::index_sequence<Ns...>, const std::tuple<Ts...> &tuple) noexcept
+    {
+        size_t seed = 0;
+        (qbs::Internal::hashCombineHelper(seed, std::get<Ns>(tuple)), ...);
+        return seed;
+    }
+
+public:
+    size_t operator()(const std::tuple<Ts...> & tuple) const noexcept
+    {
+        return helper(std::make_index_sequence<sizeof...(Ts)>(), tuple);
+    }
+};
+
+
+template<> struct hash<QStringList>
+{
+    std::size_t operator()(const QStringList &s) const noexcept
+    {
+        return qbs::Internal::hashRange(s);
+    }
+};
+
+template<> struct hash<QVariant>
+{
+    size_t operator()(const QVariant &v) const noexcept;
+};
+
+template<> struct hash<QVariantList>
+{
+    size_t operator()(const QVariantList &v) const noexcept
+    {
+        return qbs::Internal::hashRange(v);
+    }
+};
+
+template<> struct hash<QVariantMap>
+{
+    size_t operator()(const QVariantMap &v) const noexcept
+    {
+        return qbs::Internal::hashRange(v);
+    }
+};
+
+template<> struct hash<QVariantHash>
+{
+    size_t operator()(const QVariantHash &v) const noexcept
+    {
+        return qbs::Internal::hashRange(v);
+    }
+};
+
 } // namespace std
 
 QT_BEGIN_NAMESPACE
 
-uint qHash(const QStringList &list);
-uint qHash(const QProcessEnvironment &env);
+qbs::QHashValueType qHash(const QStringList &list);
+qbs::QHashValueType qHash(const QProcessEnvironment &env);
 
-#if (QT_VERSION < QT_VERSION_CHECK(5, 14, 0))
-namespace Qt {
-inline QTextStream &endl(QTextStream &stream) { return stream << QT_PREPEND_NAMESPACE(endl); }
-} // namespace Qt
-#endif
+template<typename... Args>
+qbs::QHashValueType qHash(const std::tuple<Args...> &tuple)
+{
+    return std::hash<std::tuple<Args...>>()(tuple) % std::numeric_limits<uint>::max();
+}
+
+inline qbs::QHashValueType qHash(const QVariant &v)
+{
+    return std::hash<QVariant>()(v) % std::numeric_limits<uint>::max();
+}
+
+inline qbs::QHashValueType qHash(const QVariantMap &v)
+{
+    return std::hash<QVariantMap>()(v) % std::numeric_limits<uint>::max();
+}
+
+inline qbs::QHashValueType qHash(const QVariantHash &v)
+{
+    return std::hash<QVariantHash>()(v) % std::numeric_limits<uint>::max();
+}
 
 QT_END_NAMESPACE
 
@@ -91,32 +156,20 @@ namespace qbs {
 template <class T>
 QSet<T> toSet(const QList<T> &list)
 {
-#if (QT_VERSION < QT_VERSION_CHECK(5, 14, 0))
-    return list.toSet();
-#else
     return QSet<T>(list.begin(), list.end());
-#endif
 }
 
 template<class T>
 QList<T> toList(const QSet<T> &set)
 {
-#if (QT_VERSION < QT_VERSION_CHECK(5, 14, 0))
-    return set.toList();
-#else
     return QList<T>(set.begin(), set.end());
-#endif
 }
 
 template<typename K, typename V>
 QHash<K, V> &unite(QHash<K, V> &h, const QHash<K, V> &other)
 {
-#if (QT_VERSION < QT_VERSION_CHECK(5, 15, 0))
-    return h.unite(other);
-#else
     h.insert(other);
     return h;
-#endif
 }
 
 inline void setupDefaultCodec(QTextStream &stream)
@@ -146,6 +199,51 @@ inline bool qVariantConvert(QVariant &variant, int typeId)
 #endif
 }
 
+inline QMetaType::Type qVariantType(const QVariant &v)
+{
+    return static_cast<QMetaType::Type>(
+#if (QT_VERSION >= QT_VERSION_CHECK(6, 0, 0))
+        v.metaType().id()
+#else
+        v.type()
+#endif
+    );
+}
+
+template<typename T>
+inline QVariant typedNullVariant()
+{
+    const auto metaType = QMetaType::fromType<T>();
+#if (QT_VERSION >= QT_VERSION_CHECK(6, 0, 0))
+    return QVariant(metaType, nullptr);
+#else
+    return QVariant(static_cast<QVariant::Type>(metaType.id()));
+#endif
+}
+
+inline bool qVariantsEqual(const QVariant &v1, const QVariant &v2)
+{
+    return v1.isNull() == v2.isNull() && v1 == v2;
+}
+
+inline bool qVariantMapsEqual(const QVariantMap &m1, const QVariantMap &m2)
+{
+    if (m1.size() != m2.size())
+        return false;
+    if (m1.isSharedWith(m2))
+        return true;
+
+    auto it1 = m1.cbegin();
+    auto it2 = m2.cbegin();
+    while (it1 != m1.cend()) {
+        if (it1.key() != it2.key() || !qVariantsEqual(it1.value(), it2.value()))
+            return false;
+        ++it2;
+        ++it1;
+    }
+    return true;
+}
+
 } // namespace qbs
 
 #endif // QBSQTTOOLS_H
