@@ -38,6 +38,8 @@
 ****************************************************************************/
 #include "setupprojectparameters.h"
 
+#include "buildoptions.h"
+
 #include <logging/logger.h>
 #include <logging/translator.h>
 #include <tools/buildgraphlocker.h>
@@ -45,6 +47,7 @@
 #include <tools/jsonhelper.h>
 #include <tools/profile.h>
 #include <tools/qbsassert.h>
+#include <tools/qttools.h>
 #include <tools/scripttools.h>
 #include <tools/settings.h>
 #include <tools/stringconstants.h>
@@ -90,12 +93,12 @@ public:
     mutable QVariantMap buildConfigurationTree;
     mutable QVariantMap overriddenValuesTree;
     mutable QVariantMap finalBuildConfigTree;
+    int maxJobCount = 0;
     bool overrideBuildGraphData;
     bool dryRun;
     bool logElapsedTime;
     bool forceProbeExecution;
     bool waitLockBuildGraph;
-    bool fallbackProviderEnabled = true;
     SetupProjectParameters::RestoreBehavior restoreBehavior;
     ErrorHandlingMode propertyCheckingMode;
     ErrorHandlingMode productErrorMode;
@@ -154,12 +157,14 @@ SetupProjectParameters SetupProjectParameters::fromJson(const QJsonObject &data)
     setValueFromJson(params.d->projectFilePath, data, "project-file-path");
     setValueFromJson(params.d->buildRoot, data, "build-root");
     setValueFromJson(params.d->settingsBaseDir, data, "settings-directory");
+    setValueFromJson(params.d->maxJobCount, data, "max-job-count");
+    if (params.maxJobCount() <= 0)
+        params.setMaxJobCount(BuildOptions::defaultMaxJobCount());
     setValueFromJson(params.d->overriddenValues, data, "overridden-properties");
     setValueFromJson(params.d->dryRun, data, "dry-run");
     setValueFromJson(params.d->logElapsedTime, data, "log-time");
     setValueFromJson(params.d->forceProbeExecution, data, "force-probe-execution");
     setValueFromJson(params.d->waitLockBuildGraph, data, "wait-lock-build-graph");
-    setValueFromJson(params.d->fallbackProviderEnabled, data, "fallback-provider-enabled");
     setValueFromJson(params.d->environment, data, "environment");
     setValueFromJson(params.d->restoreBehavior, data, "restore-behavior");
     setValueFromJson(params.d->propertyCheckingMode, data, "error-handling-mode");
@@ -372,6 +377,27 @@ void SetupProjectParameters::setSettingsDirectory(const QString &settingsBaseDir
     d->settingsBaseDir = settingsBaseDir;
 }
 
+/*!
+ * \brief Returns the maximum number of threads to employ when resolving the project.
+ * If the value is not valid (i.e. <= 0), a sensible one will be derived from the number of
+ * available processor cores.
+ * The default is 0.
+ * \sa BuildOptions::defaultMaxJobCount
+ */
+int SetupProjectParameters::maxJobCount() const
+{
+    return d->maxJobCount;
+}
+
+/*!
+ * \brief Controls how many threads to employ when resolving the project.
+ * A value <= 0 leaves the decision to qbs.
+ */
+void SetupProjectParameters::setMaxJobCount(int jobCount)
+{
+    d->maxJobCount = jobCount;
+}
+
 /*!
  * Returns the overridden values of the build configuration.
  */
@@ -490,7 +516,7 @@ ErrorInfo SetupProjectParameters::expandBuildConfiguration()
     QVariantMap expandedConfig = expandedBuildConfiguration(profile, configurationName(), &err);
     if (err.hasError())
         return err;
-    if (d->buildConfiguration != expandedConfig) {
+    if (!qVariantMapsEqual(d->buildConfiguration, expandedConfig)) {
         d->buildConfigurationTree.clear();
         d->buildConfiguration = expandedConfig;
     }
@@ -594,22 +620,6 @@ void SetupProjectParameters::setWaitLockBuildGraph(bool wait)
     d->waitLockBuildGraph = wait;
 }
 
-/*!
- * \brief Returns true if qbs should fall back to pkg-config if a dependency is not found.
- */
-bool SetupProjectParameters::fallbackProviderEnabled() const
-{
-    return d->fallbackProviderEnabled;
-}
-
-/*!
- * Controls whether to fall back to pkg-config if a dependency is not found.
- */
-void SetupProjectParameters::setFallbackProviderEnabled(bool enable)
-{
-    d->fallbackProviderEnabled = enable;
-}
-
 /*!
  * \brief Gets the environment used while resolving the project.
  */
