@@ -110,6 +110,7 @@ public:
     QBS_STRING_CONSTANT(installDirProperty, "installDir")
     QBS_STRING_CONSTANT(installSourceBaseProperty, "installSourceBase")
     QBS_STRING_CONSTANT(isEnabledKey, "is-enabled")
+    QBS_STRING_CONSTANT(isEagerProperty, "isEager")
     QBS_STRING_CONSTANT(jobCountProperty, "jobCount")
     QBS_STRING_CONSTANT(jobPoolProperty, "jobPool")
     QBS_STRING_CONSTANT(lengthProperty, "length")
@@ -117,6 +118,7 @@ public:
     QBS_STRING_CONSTANT(locationKey, "location")
     QBS_STRING_CONSTANT(messageKey, "message")
     QBS_STRING_CONSTANT(minimumQbsVersionProperty, "minimumQbsVersion")
+    QBS_STRING_CONSTANT(moduleVar, "module")
     QBS_STRING_CONSTANT(moduleNameProperty, "moduleName")
     QBS_STRING_CONSTANT(modulePropertiesKey, "module-properties")
     QBS_STRING_CONSTANT(moduleProviders, "moduleProviders")
@@ -143,6 +145,7 @@ public:
     static const QString &profilesProperty() { return profiles(); }
     QBS_STRING_CONSTANT(productTypesProperty, "productTypes")
     QBS_STRING_CONSTANT(productsKey, "products")
+    QBS_STRING_CONSTANT(qbsModuleProviders, "qbsModuleProviders")
     QBS_STRING_CONSTANT(qbsSearchPathsProperty, "qbsSearchPaths")
     QBS_STRING_CONSTANT(referencesProperty, "references")
     QBS_STRING_CONSTANT(recursiveProperty, "recursive")
@@ -167,6 +170,7 @@ public:
 
     QBS_STRING_CONSTANT(importScopeNamePropertyInternal, "_qbs_importScopeName")
     QBS_STRING_CONSTANT(modulePropertyInternal, "__module")
+    QBS_STRING_CONSTANT(dataPropertyInternal, "_qbs_data")
     QBS_STRING_CONSTANT(qbsSourceDirPropertyInternal, "_qbs_sourceDir")
     static const char *qbsProcEnvVarInternal() { return "_qbs_procenv"; }
 
